using System;
using Autofac;
using FitConnect.Encryption;
using FitConnect.Models;
using FitConnect.Models.v1.Api;
using FluentAssertions;
using MockContainer;
using NUnit.Framework;
using SecurityEventToken = FitConnect.Models.SecurityEventToken;

namespace FluentApiTest;

[TestFixture]
public class SecurityEventTokenTests {
    private const string rejectSubmission =
        SecurityEventToken.RejectSubmissionSchema;

    private FitEncryption _encryption = null!;

    private const string acceptSubmission =
        SecurityEventToken.AcceptSubmissionSchema;

    [SetUp]
    public void Setup() {
        var container = Container.Create();
        _encryption = new FitEncryption(container.Resolve<KeySet>(), null);
    }


    [Test]
    public void CreateJwt_AcceptSubmission() {
        var token = _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            acceptSubmission,
            null
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(acceptSubmission);
        decoded.EventType.Should().Be(EventType.Accept);
    }

    [Test]
    public void CreateJwt_Reject_WithEncryptionIssue() {
        var token = _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            rejectSubmission,
            new[] { Problems.EncryptionIssue }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithMissingSchema() {
        var token = _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            rejectSubmission,
            new[] { Problems.MissingSchema }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);

        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithSchemaViolation() {
        var token = _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            rejectSubmission,
            new[] { Problems.SchemaViolation }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithSyntaxViolation() {
        var token = _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            rejectSubmission,
            new[] { Problems.SyntaxViolation }
        );
        Console.WriteLine(token);

        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithUnsupportedSchema() {
        var token = _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            rejectSubmission,
            new[] { Problems.UnsupportedSchema }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithIncorrectAuthenticationTag() {
        var token = _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            rejectSubmission,
            new[] { Problems.IncorrectAuthenticationTag }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithCustomProblem() {
        var token = _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            rejectSubmission,
            new[] { new Problems() { Description = "A real big issue" } }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }
}
