using System.Security.Claims;
using FitConnect.Models.v1.Api;
using Microsoft.IdentityModel.JsonWebTokens;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace FitConnect.Models;

public enum EventType {
    NotSet,
    Create,
    Submit,
    Notify,
    Forward,
    Reject,
    Accept,
    Delete
}

public class SecurityEventToken {
    public const string CreateSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/create-submission";

    public const string SubmitSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/submit-submission";

    public const string NotifySubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/notify-submission";

    public const string ForwardSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/forward-submission";

    public const string RejectSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/reject-submission";

    public const string AcceptSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/accept-submission";

    public const string DeleteSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/delete-submission";

    public SecurityEventToken(string jwtEncodedString) {
        Token = new JsonWebToken(jwtEncodedString);
        EventType = DecodeEventType(Token.Claims);
        var iat = Token.Claims.FirstOrDefault(c => c.Type == "iat").Value;
        if (long.TryParse(iat, out var timeEpoch))
            EventTime = DateTime.UnixEpoch.AddSeconds(timeEpoch);
    }

    public DateTime EventTime { get; set; }
    public EventType EventType { get; set; }
    public List<Problems>? Problems { get; set; }
    public Events? Event { get; set; }

    public object? Payload { get; set; }
    public JsonWebToken Token { get; set; }

    private EventType DecodeEventType(IEnumerable<Claim> claims) {
        var eventsClaim = claims.FirstOrDefault(c => c.Type == "events");
        if (eventsClaim == null) return EventType.NotSet;

        var events = JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(eventsClaim.Value);
        Payload = JsonConvert
            .DeserializeObject<dynamic>(
                events?.Values?.FirstOrDefault()?.ToString());

        var contentData = JsonConvert.DeserializeObject(eventsClaim.Value);

        if (eventsClaim.Value.Contains(
                CreateSubmissionSchema))
            return EventType.Create;

        if (eventsClaim.Value.Contains(
                SubmitSubmissionSchema))
            return EventType.Submit;

        if (eventsClaim.Value.Contains(
                NotifySubmissionSchema))
            return EventType.Notify;
        if (eventsClaim.Value.Contains(
                ForwardSubmissionSchema))
            return EventType.Forward;
        if (eventsClaim.Value.Contains(
                RejectSubmissionSchema)) {
            Problems = GetProblems(events?.Values?.FirstOrDefault()?.ToString() ?? "");
            return EventType.Reject;
        }


        if (eventsClaim.Value.Contains(AcceptSubmissionSchema))
            return EventType.Accept;


        if (eventsClaim.Value.Contains(
                DeleteSubmissionSchema))
            return EventType.Delete;

        return EventType.NotSet;
    }

    private static List<Problems> GetProblems(string payload) {
        if (payload == "") return new List<Problems>();
        return (JsonConvert.DeserializeObject<dynamic>(payload)?.problems as JArray)
            ?.Select(j => j as dynamic).ToList()
            ?.Select(p => new Problems {
                Title = p.Title,
                Description = p.Description,
                Items = p.Items,
                Type = p.Type,
                Detail = p.Detail,
                Instance = p.Instance
            })
            ?.ToList() ?? new List<Problems>();
    }
}
