using System.Text.Encodings.Web;
using System.Text.Json;
using Newtonsoft.Json;
using FitConnect.Services.Interfaces;
using FitConnect.Services.Models.v1.Case;
using FitConnect.Services.Models.v1.Submission;
using Microsoft.Extensions.Logging;
using JsonSerializer = System.Text.Json.JsonSerializer;


namespace FitConnect.Services;

internal class SubmissionService : RestCallService, ISubmissionService {
    private readonly ILogger? _logger;
    private readonly IOAuthService _oAuthService;

    public SubmissionService(string baseUrl, IOAuthService oAuthService,
        string version = "v1", ILogger? logger = null) : base(
        $"{baseUrl}/{version}", logger) {
        _oAuthService = oAuthService;
        _logger = logger;
    }

    /// <summary>
    ///     <para>@PostMapping("/submissions")</para>
    /// </summary>
    /// <param name="submissionDto">RequestBody</param>
    /// <returns></returns>
    public SubmissionCreatedDto CreateSubmission(CreateSubmissionDto submissionDto) {
        _oAuthService.EnsureAuthenticated();
        var result = RestCall<SubmissionCreatedDto>(
            "/submissions",
            HttpMethod.Post,
            JsonConvert.SerializeObject(submissionDto)).Result;

        if (result == null)
            throw new ArgumentException("Submission creation failed");

        return result;
    }


    /// <summary>
    ///     <para>
    ///         @PutMapping(value = "/submissions/{submissionId}/attachments/{attachmentId}", consumes =
    ///         "application/jose")
    ///     </para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <param name="attachmentId">PathVariable</param>
    /// <param name="encryptedAttachmentContent">RequestBody</param>
    /// <returns></returns>
    public bool AddSubmissionAttachment(string submissionId, string attachmentId,
        string encryptedAttachmentContent) {
        _oAuthService.EnsureAuthenticated();
        var result = RestCallForString(
            $"/submissions/{submissionId}/attachments/{attachmentId}",
            HttpMethod.Put,
            encryptedAttachmentContent,
            "application/jose").Result;
        return true;
    }

    /// <summary>
    ///     <para>@PutMapping(value = "/submissions/{submissionId}", consumes = "application/json") </para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <param name="submitSubmission">RequestBody</param>
    /// <returns></returns>
    public async Task<SubmissionReducedDto?> SubmitSubmission(string submissionId,
        SubmitSubmissionDto submitSubmission) {
        _oAuthService.EnsureAuthenticated();
        var body = JsonConvert.SerializeObject(submitSubmission, new JsonSerializerSettings() {
            NullValueHandling = NullValueHandling.Ignore,
            Formatting = Formatting.Indented
        });

        var result = await RestCall<SubmissionReducedDto>($"/submissions/{submissionId}",
            HttpMethod.Put,
            body);

        return result;
    }


    /// <summary>
    ///     <para>@GetMapping("/submissions")</para>
    /// </summary>
    /// <param name="destinationId">RequestParam</param>
    /// <param name="offset">RequestParam</param>
    /// <param name="limit">RequestParam</param>
    /// <returns></returns>
    public async Task<SubmissionsForPickupDto> ListSubmissions(string? destinationId,
        int offset = 0,
        int limit = 100) {
        _oAuthService.EnsureAuthenticated();
        var url = destinationId != null
            ? $"/submissions?destinationId={destinationId}&offset={offset}&limit={limit}"
            : $"/submissions?offset={offset}&limit={limit}";
        return RestCall<SubmissionsForPickupDto>(url, HttpMethod.Get).Result ??
               new SubmissionsForPickupDto();
    }

    /// <summary>
    ///     <para>@GetMapping("/submissions/{submissionId}")</para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <returns></returns>
    public SubmissionDto GetSubmission(string submissionId) {
        _oAuthService.EnsureAuthenticated();
        var result =
            RestCall<SubmissionDto>($"/submissions/{submissionId}", HttpMethod.Get).Result ??
            new SubmissionDto();

        return result;
    }


    /// <summary>
    ///     <para>
    ///         @GetMapping(value = "/submissions/{submissionId}/attachments/{attachmentId}", produces =
    ///         "application/jose")
    ///     </para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <param name="attachmentId">PathVariable</param>
    /// <returns></returns>
    public string GetAttachment(string submissionId, string attachmentId) {
        _oAuthService.EnsureAuthenticated();
        var result = RestCallForString($"/submissions/{submissionId}/attachments/{attachmentId}",
            HttpMethod.Get,
            null,
            "application/jose",
            "application/jose").Result;

        return result;
    }

    public string GetKey(string keyId) {
        throw new NotImplementedException();
    }

    public async Task<List<string>> GetStatusForSubmissionAsync(string caseId) {
        _oAuthService.EnsureAuthenticated();
        var events = await RestCall<EventLogDto>($"/cases/{caseId}/events", HttpMethod.Get);
        return events.EventLog.ToList();
    }

    public async Task GetValidationJwk() {
        throw new NotImplementedException();
    }
}
